


--
-- Created by IntelliJ IDEA.
-- User: sean
-- Date: 15-11-27
-- Time: 下午3:06
-- To change this template use File | Settings | File Templates.
--

local JSON = require "ejoysdk_lua.ejoysdk_json"
local ET = require "ejoysdk_lua.ejoysdk_topic"

-- async call type
local ACT_HTTP_GET = 'HTTP_GET'
local ACT_HTTP_POST = 'HTTP_POST'
local ACT_MODAL_OPEN = 'MODAL_OPEN'
local ACT_TIMER_ONCE = 'TIMER_ONCE'
local ACT_MEDIA_START_RECORD = 'MEDIA_START_RECORD'
local ACT_MEDIA_STOP_RECORD = 'MEDIA_STOP_RECORD'
local ACT_MEDIA_START_PLAY = 'MEDIA_START_PLAY'
local ACT_MEDIA_STOP_PLAY = 'MEDIA_STOP_PLAY'
local ACT_MEDIA_DELETE = 'MEDIA_DELETE'
-- local ACT_GOOGLE_PLAY_PURCHASE = 'GOOGLE_PLAY_PURCHASE'

-- sync call type
local CT_KEYSTORE_GET = 'KEYSTORE_GET'
local CT_MEDIA_RECORD_DIR = 'MEDIA_RECORD_DIR'

-- invoke
local IVK_LOG_PARAMS = 'LOG_PARAMS'
local IVK_KEYSTORE_SET = 'KEYSTORE_SET'
local IVK_KEYSTORE_DELETE = 'KEYSTORE_DELETE'
local IVK_KEYSTORE_CLEAR = 'KEYSTORE_CLEAR'
local IVK_WEBVIEW_OPEN = 'WEBVIEW_OPEN'
local IVK_WEBVIEW_CLOSE = 'WEBVIEW_CLOSE'
local IVK_WEBVIEW_IS_OPENED = 'WEBVIEW_IS_OPENED'
local IVK_TOAST_OPEN = 'TOAST_OPEN'
local IVK_SYSINFO_IDFA = 'SYSINFO_IDFA'
local IVK_SYSINFO_OPENGL = 'SYSINFO_OPENGL'
local IVK_SYSINFO_MEMORY = 'SYSINFO_MEMORY'
local IVK_SYSINFO_SCREEN = 'SYSINFO_SCREEN'
local IVK_SYSINFO_OPEN_URL = 'SYSINFO_OPEN_URL'
local IVK_GOTO_APPLICATION_SETTINGS = 'GOTO_APPLICATION_SETTINGS'


local M = {}
local cbs = {}
local cb_id = math.random(1000, 9999)

M.PLATFORM = {
    OS = "Android",
    HTTP_UA = "EjoySDK-http-client/0.1 (Linux; Android)"
}

M.JAVA_CALL_STATIC_CLASS = 'com/ejoy/ejoysdk/LuaCall'

function M.async_call(type, params, chunk, cb, opts)
    chunk = chunk or ''

    local id = cb_id
    cb_id = cb_id + 1
    cbs[id] = cb

    if opts and opts.timeout and opts.timeout_cb then
        M.Timer.once(opts.timeout, function()
            if cbs[id] then
                cbs[id] = nil
                opts.timeout_cb()
            end
        end)
    end

    local json_str = JSON:encode(params)
    return _ejoysdk.async_call(M.JAVA_CALL_STATIC_CLASS, type, id, json_str, chunk)
end

local function async_callback(id, json_str, chunk)
    local cb = cbs[id]
    if cb then
        local resp = JSON:decode(json_str)
        cbs[id] = nil
        cb(resp, chunk)
    end
end

_ejoysdk.register_cb("ASYNC_CALL", async_callback)

function M.sync_call(type, params, chunk)
    params = params or {}
    chunk = chunk or ''
    local json_str = JSON:encode(params)
    local json_ret = _ejoysdk.sync_call(M.JAVA_CALL_STATIC_CLASS, type, json_str, chunk)
    return json_ret and JSON:decode(json_ret)
end

function M.invoke(type, params, chunk)
    params = params or {}
    chunk = chunk or ''
    local json_str = JSON:encode(params)
    return _ejoysdk.invoke(M.JAVA_CALL_STATIC_CLASS, type, json_str, chunk)
end

function M.register_async_cb(type, cb)
    _ejoysdk.register_cb(type, cb)
end

function M.log(params)
    return M.invoke(IVK_LOG_PARAMS, params)
end
function M.printl(content)
    M.log({msg= content})
end

local HTTP_EVENT = 'HTTP_EVENT'
local http_progress_cbs = {}

_ejoysdk.register_cb(HTTP_EVENT, function(cbid, js_str, _chunk)
    local params  = http_progress_cbs[cbid]
    if params then
        local resp = JSON:decode(js_str)
        if resp.received == -1 then
            http_progress_cbs[cbid] = nil
        else
            params.progress(params.url, params.file, resp.received, resp.total)
        end
    end
end)

local HTTP = {}
M.HTTP = HTTP

function HTTP.process_get(url, params, cb)
    params.headers = HTTP.check_and_update_headers(params.headers)
    local progress = params.progress
    params.progress = nil
    local cbid = M.async_call(ACT_HTTP_GET, {url= url, params= params}, '', function(info, body)
        info.headers = HTTP.Header.New(info.headers)
        if not params.raw_body then
            body = HTTP.decode_by_content_type(params.acceptable, info.headers, body)
        end
        info.body = body
        cb(info)
    end, {
        timeout=params.timeout,
        timeout_cb=function()
            cb({status=0})
        end
    })
    if params.file and progress then
        http_progress_cbs[cbid] = {url=url, file=params.file, progress=progress}
    end
end

function HTTP.process_post(url, params, content_type, body, cb)
    params.headers = HTTP.check_and_update_headers(params.headers)
    if params.headers['Content-Type'] then
        assert(params.headers['Content-Type'] == content_type)
    end

    params['Content-Type'] = content_type
    body = HTTP.smart_content(body, content_type)

    local progress = params.progress
    params.progress = nil
    local cbid = M.async_call(ACT_HTTP_POST, {url = url, params = params}, body, function(info, resp_body)
        info.headers = HTTP.Header.New(info.headers)
        if not params.raw_body then
            resp_body = HTTP.decode_by_content_type(params.acceptable, info.headers, resp_body)
        end
        info.body = resp_body
        cb(info)
    end, {
        timeout=params.timeout,
        timeout_cb = function()
            cb({status=0})
        end
    })
    if params.file and progress then
        http_progress_cbs[cbid] = {url=url, file=params.file, progress=progress}
    end
end

local KEYSTORE_KEY = 'ejoysdk'
local KeyStore = {}
M.KeyStore = KeyStore

function KeyStore.get(key)
    return M.sync_call(CT_KEYSTORE_GET, {service=KEYSTORE_KEY, key=key}).value
end

function KeyStore.set(key, value)
    if value == nil then
        return
    end
    M.invoke(IVK_KEYSTORE_SET, {service=KEYSTORE_KEY, key=key, value=value})
end

function KeyStore.delete(key)
    M.invoke(IVK_KEYSTORE_DELETE, {service=KEYSTORE_KEY, key=key})
end

function KeyStore.clear()
    M.invoke(IVK_KEYSTORE_CLEAR, {service=KEYSTORE_KEY})
end

local WebView = {}
M.WebView = WebView

function WebView.open(url, injection, option)
    injection = injection or {}
    if not injection['.ejoy.com'] then
        injection['.ejoy.com'] = {}
    end
    option = option or {}
    return M.invoke(IVK_WEBVIEW_OPEN, {url=url, injection=injection, option=option})
end

function WebView.close()
    return M.invoke(IVK_WEBVIEW_CLOSE)
end

function WebView.is_opened()
    local result = M.sync_call(IVK_WEBVIEW_IS_OPENED)
    return result.isOpened == true
end

local WEBVIEW_EVENT = 'WEBVIEW_EVENT'
local WEBVIEW_JSARGS_EVENT = 0;
local WEBVIEW_CLOSE_EVENT = 1;
_ejoysdk.register_cb(WEBVIEW_EVENT, function(cbid, js_str, _chunk)
    local value = JSON:decode(js_str)
    if cbid == WEBVIEW_JSARGS_EVENT then
        if value.args.type == 'oauthUri' then
            ET.publish('logindone', value);
            return
        end
    end
    if cbid == WEBVIEW_CLOSE_EVENT then
        ET.publish('webview_close', value);
        return
    end
end)

local APP_EVENT = 'APP_EVENT'

local _app_event_handler = {
    [0] = function(value, _chunk)
        ET.publish('urlopen', value.url)
    end,
    [1] = function()
        ET.publish("app_on_stop")
    end,
    [2] = function()
        ET.publish('backpress')
    end,
}
_ejoysdk.register_cb(APP_EVENT, function(cbid, js_str, chunk)
    local handler = _app_event_handler[cbid]
    if handler then
        local value = JSON:decode(js_str)
        handler(value, chunk)
    end
end)

local Modal = {}
M.Modal = Modal

function Modal.open(title, option, cb)
    option = option or {}
    cb = cb or function() end
    M.async_call(ACT_MODAL_OPEN, {title= title, option= option}, '', function(info, _body)
        cb(info.index);
    end)
end

function Modal.alert(title, message, cb)
    local option = {
        message= message,
        buttons= {'确定'}
    }
    local cb_wrap = function(_index) cb() end
    Modal.open(title, option, cb_wrap);
end

function Modal.confirm(title, message, cb)
    local option = {
        message= message,
        buttons= {'取消', '确定'}
    }
    local cb_wrap = function(index)
        cb(index ~= 0);
    end
    Modal.open(title, option, cb_wrap);
end

local Toast = {}
M.Toast = Toast

function Toast.show(message, option)
    option = option or {}
    return M.invoke(IVK_TOAST_OPEN, {message= message, option=option})
end

function Toast.hide()
    return M.invoke(IVK_TOAST_OPEN, {message= '', option={}})
end

local Sysinfo = {}
M.Sysinfo = Sysinfo

function Sysinfo.idfa()
    local result = M.sync_call(IVK_SYSINFO_IDFA)
    return result.idfa
end

function Sysinfo.screen()
    return M.sync_call(IVK_SYSINFO_SCREEN)
end

function Sysinfo.memory()
    return M.sync_call(IVK_SYSINFO_MEMORY)
end

function Sysinfo.opengl()
    return M.sync_call(IVK_SYSINFO_OPENGL)
end

function Sysinfo.open_url(url)
    return M.invoke(IVK_SYSINFO_OPEN_URL, {url= url})
end

local Timer = {}
M.Timer = Timer

function Timer.once(interval, cb)
    M.async_call(ACT_TIMER_ONCE, {interval=interval}, '', function(_info, _body)
        cb();
    end)
end

function M.tick(once)
    repeat
        local cb_type, cbid, msg, chunk = _ejoysdk.tick(M.JAVA_CALL_STATIC_CLASS)
        if cb_type then
            local cb = _ejoysdk.get_register_cb(cb_type)
            if cb then
                cb(cbid, msg, chunk)
            end
        else
            return false
        end
    until once == true
    return true
end

function M.on_stop()
    ET.publish("app_on_stop")
end

local Media = {
    DEFAULT_MAX_FILESIZE = 1024 * 1024 * 5,
    DEFAULT_MAX_DURATION = 60 * 1000 * 3,
    DEFAULT_SAMPLING_RATE = 8000,
    DEFAULT_ENCODING_BIT_RATE = 16,
    DEFAULT_CHANNEL = 1,
}
M.Media = Media

function Media.start_record(opt, cb)
    opt = opt or {}

    local max_filesize = Media.DEFAULT_MAX_FILESIZE
    if opt.max_filesize and opt.max_filesize <= max_filesize then
        max_filesize = opt.max_filesize
    end
    local max_duration = Media.DEFAULT_MAX_DURATION
    if opt.max_duration and opt.max_duration <= max_duration then
        max_duration = opt.max_duration
    end

    local params = {
        filename = opt.filename or 'noname',
        format = opt.format or 'amr',
        max_filesize = max_filesize,
        max_duration = max_duration,
        sampling_rate = opt.sampling_rate or Media.DEFAULT_SAMPLING_RATE,
        encoding_bit_rate = opt.encoding_bit_rate or Media.DEFAULT_ENCODING_BIT_RATE,
        channel = opt.channel or Media.DEFAULT_CHANNEL,
        show_permission_request = opt.show_permission_request or true,
    }

    if params.format == 'amr' then
        if params.sampling_rate ~= 8000 then
            error('when format is amr, sampling_rate should be 8000');
            return
        end
        if params.encoding_bit_rate ~= 16 then
            error('when format is amr, encoding_bit_rate should be 16');
            return
        end
        if params.channel ~= 1 then
            error('when format is amr, channel should be 1');
            return
        end
    end

    M.async_call(ACT_MEDIA_START_RECORD, params, '', cb)
end

function Media.stop_record(opt, cb)
    local params = opt or {}
    local cb_wrap = function(info, body)
        info.bytes = body
        cb(info)
    end
    M.async_call(ACT_MEDIA_STOP_RECORD, params, '', cb_wrap)
end

function Media.start_play(opt, cb)
    opt = opt or {}

    local format = 'amr'
    if opt.format ~= 'auto' then
        format = opt.format
    end

    local params = {
        filename = opt.filename or 'noname',
        format = format,
    }
    M.async_call(ACT_MEDIA_START_PLAY, params, '', cb)
end

function Media.stop_play(opt, cb)
    local params = opt or {}
    M.async_call(ACT_MEDIA_STOP_PLAY, params, '', cb)
end

function Media.delete(opt, cb)
    local params = opt or {}
    M.async_call(ACT_MEDIA_DELETE, params, '', cb)
end

function Media.get_record_dir()
    local ret = M.sync_call(CT_MEDIA_RECORD_DIR)
    return ret.path
end

function M.goto_application_settings()
    M.invoke(IVK_GOTO_APPLICATION_SETTINGS)
end

return M
