local E = require "ejoysdk_lua.ejoysdk"
local EC = require "ejoysdk_lua.ejoysdk_config"
local ET = require "ejoysdk_lua.ejoysdk_topic"

local M = {}
M.TRACE_EVENT = 'trace_event'

local vendor_cache
local inited = false

local URLS = {}
local url_items = {
    partner = 'trace/partner',
    boot = 'trace/boot',
    log = 'trace/tracelog',
    ticket = 'ann/v2/ticket/',
    detail = 'ann/v2/detail/',
}

local KS_VENDERS_KEY = 'VENDERS'
local function get_vendors_stored()
    local v = E.KeyStore.get(KS_VENDERS_KEY)
    if v then
        return E.Utils.split_string(v, ':')
    end
end

local function save_vendors(vendors)
    if #vendors > 0 then
        local v = table.concat(vendors, ':')
        E.KeyStore.set(KS_VENDERS_KEY, v)
    end
end

--[[
-- opt 是一个table，有各种开关
-- vendor: true 向服务器同步自己的idfa， 获取广告渠道
--]]
function M.init(opt)
    if inited then
        return
    end

    local url_prefix = E.CONFIG.get_config("launcher") .. '/'
    for k, v in pairs(url_items) do
        URLS[k] = url_prefix .. v
    end

    if opt.vendor then
        local vendors = get_vendors_stored()
        if not vendors then
            M.get_vendors(function(succ)
                if succ then
                    inited = true
                end
            end)
        else
            inited = true
            vendor_cache = vendors
        end
    end
end

function M.event(tag, detail)
    if detail == nil then
        detail = tag
        tag = 'boot'
    end

    local idfa = E.Sysinfo.idfa()
    local params = {
        tag = tag,
        uuid=idfa,
        timestamp=os.time(),
        detail = detail,
    }
    E.HTTP.post(URLS.log, {acceptable = E.HTTP.CT_JSON}, E.HTTP.CT_JSON, params, function(resp)
        if resp.status ~= 200 or resp.body.code ~= 200 then
            E.log({error='send event error', resp=resp})
        end
    end)
end

-- 可以直接用 topic 来记录 event
ET.subscribe(M.TRACE_EVENT, M.event)


function M.ticket(ann_type, tags, cb)
    local url = URLS.ticket .. ann_type
    local tags_str = nil
    if type(tags) == 'table' then
        tags_str = table.concat(tags, ",")
    elseif type(tags) == 'string' then
        tags_str = tags
    end

    local params = {
        zone=EC.get_config('zone'),
        lang=EC.get_config('lang'),
        tags=tags_str,
    }
    url = E.HTTP.url_query(url, params)
    E.HTTP.get(url, {acceptable = E.HTTP.CT_JSON}, function(resp)
        if resp.status == 200 then
            local body = resp.body
            cb(true, body.hash, body.time)
        else
            cb(false)
        end
    end)
end

function M.detail(ticket, cb)
    local url = URLS.detail .. ticket
    E.HTTP.get(url, {raw_body=true}, function(resp)
        if resp.status == 200 then
            cb(true, resp.body, ticket)
        else
            cb(false)
        end
    end)
end

function M.vendors()
    return vendor_cache
end

-- internal
--
local function update_vendor_cache(vendors)
    save_vendors(vendors)
    vendor_cache = vendors
end

function M.get_vendors(cb)
    local idfa = E.Sysinfo.idfa()
    local timestamp = os.time()
    local params = {
        product = E.CONFIG.get_config("product"),
        idfa = idfa;
        timestamp = timestamp
    }
    E.HTTP.post(URLS.partner, {acceptable = E.HTTP.CT_JSON}, E.HTTP.CT_URLENCODED, params, function(resp)
        if resp.status == 200 then
            if resp.body.code == 200 then
                E.log(resp.body)
                update_vendor_cache(resp.body.data)
                cb(true, resp.body.data)
                return
            end
        end
        cb(false)
    end)
end

return M
