local M = {}
local dispatcher = {}

function M.subscribe(topic, cb)
    local handlers = dispatcher[topic]
    if not handlers then
        handlers = {}
        dispatcher[topic] = handlers
    end

    for _, handler in ipairs(handlers) do
        if handler == cb then
            return
        end
    end
    handlers[#handlers + 1] = cb
end

function M.unsubscribe(topic, cb)
    local handlers = dispatcher[topic]
    if handlers then
        local new = {}
        for _, handler in ipairs(handlers) do
            if cb ~= handler then
                new[#new + 1] = handler
            end
        end
        dispatcher[topic] = new
    end
end

function M.publish(topic, ...)
    local handlers = dispatcher[topic]
    if handlers then
        for _, cb in ipairs(handlers) do
            local succ, err = pcall(cb, ...)
            if not succ then
                _ejoysdk.log("error topic " .. tostring(topic) .. ': ' .. tostring(err))
            end
        end
    end
end

return M
