local E = require "ejoysdk_lua.ejoysdk"

local EVV = require "ejoysdk_lua.vendors.vendor"
local EUS = require "ejoysdk_lua.vendors.unisdk"

local M = {}

function M.init(vendors, listeners)
    M.VENDORS = {
        EJOY = require "ejoysdk_lua.vendors.ejoy",
        GOOGLE = require "ejoysdk_lua.vendors.google_play",
        APPLE = require "ejoysdk_lua.vendors.apple",
        FACEBOOK = require "ejoysdk_lua.vendors.facebook",
        TEST = require "ejoysdk_lua.vendors.testsdk",
        ALIPAY = require "ejoysdk_lua.vendors.alipay",
        WEIXIN = require "ejoysdk_lua.vendors.weixin",
        ZLONG = require "ejoysdk_lua.vendors.zilong",
        XIAOMI = require "ejoysdk_lua.vendors.xiaomi",
        UC = require "ejoysdk_lua.vendors.uc",
        XG = require "ejoysdk_lua.vendors.xg",
    }

    for k, v in pairs(M.VENDORS) do
        if v.init then
            local auth_listener = function(...)
                listeners.auth_listener(k, ...)
            end
            local pay_listener = function(...)
                listeners.pay_listener(k, ...)
            end
            local switch_listener = function(...)
                listeners.switch_listener(k, ...)
            end
            local vopt = vendors[k]
            if vopt then
                vopt.auth_listener = auth_listener
                vopt.pay_listener = pay_listener
                vopt.switch_listener = switch_listener
                v.init(vopt)
            end
        end
    end
end

function M.get_vendor_name(outoutsource)
    return outoutsource.with or outoutsource.platform
end

function M.get(vendor)
    return M.VENDORS[vendor]
end

M.ABILITY = EVV.ABILITY

--[
--获取当前包支持的渠道列表
--
--@params ability:  ACCOUNT, PAY ...
--]
function M.get_vendors(ability)
    assert(M.ABILITY[ability], tostring(ability) .. " not found")

    local base = EUS.get_sdk(ability).sdks
    local os = E.CONFIG.get_config('os')

    if os == 'ios' then
        table.insert(base, 'APPLE')
    end

    if ability == M.ABILITY.ACCOUNT then
        table.insert(base, 'EJOY')
    end

    return base
end

return M
