local E = require "ejoysdk_lua.ejoysdk"
local ET = require "ejoysdk_lua.ejoysdk_topic"

local M = {}
local nop_handler = function() end
local cb_handler = nop_handler
local fb_token = nil


local function oauth_url()
    return 'https://www.facebook.com/dialog/oauth'
end

local function fb_catch_token_url(env)
    local os = E.CONFIG.get_config('os')
    local url_base = E.CONFIG.get_config('launcher')
    return url_base .. '/oauth/facebook/' .. os .. '/' .. env
end

local function on_url(url)
    local ret = E.HTTP.parse(url)
    if ret.host == 'sdk.ejoy.com' and ret.query then
        local query = ret.query
        if query._platform == 'facebook' and query.access_token and query.state then
            fb_token = query.access_token
            cb_handler(true, query.access_token, query.state)
        else
            fb_token = nil
            if query.error then
                -- log error msg
                E.log(query)
            end
            cb_handler(false)
        end
    end
end

local function on_login_done(value)
    on_url(value.args.uri);
end

local function on_webview_close(_value)
    cb_handler(false)
end

function M.init(opt)
    assert(opt, "options is nil")
    assert(opt.facebook_client_id, "facebook_client_id not found")
    assert(opt.type, "login type not found")
    M.opt = opt
    ET.subscribe('urlopen', on_url)
    ET.subscribe('logindone', on_login_done)
    ET.subscribe('webview_close', on_webview_close)
end

function M.has_token()
    return fb_token
end

function M.login(cb, _uuid)
    local viewer_type = M.opt.type

    local state = _ejoysdk_crypt.base64encode(_ejoysdk_crypt.randomkey())
    local query = {
        response_type = 'token',
        client_id = M.opt.facebook_client_id,
        redirect_uri = fb_catch_token_url(viewer_type),
        scope = 'email',
        state = state,
    }
    local url = E.HTTP.url_query(oauth_url(), query)
    cb_handler = function(succ, token, new_state)
        cb_handler = nop_handler
        if not succ or new_state ~= state then
            fb_token = nil
            cb(false)
            return
        end
        cb(succ, token)
    end

    if viewer_type == 'browser' then
        E.Sysinfo.open_url(url)
    else
        E.WebView.open(url, {
            [".ejoy.com"] = {}
        })
    end
end

return M
