local ET = require "ejoysdk_lua.ejoysdk_topic"
local EG = require "ejoysdk_lua.ejoysdk_gangplank"
local UNI = require "ejoysdk_lua.vendors.unisdk"
local Vendor = require "ejoysdk_lua.vendors.vendor"

local CHANNEL = "UC"

local ASYNC_EXIT = "ASYNC_EXIT"
local CAST_SUBMIT_ROLE_DATA = "CAST_SUBMIT_ROLE_DATA"

local M = Vendor:Inherit(CHANNEL)
local auth_listener = nil
local pay_listener = nil
local inited = false
local product_infos = {}

local login_handler = function()
    if inited then
        return
    end

    _ejoysdk.log("get uc product info")

    EG.product_infos_base(CHANNEL, function(ejoy_succ, infos)
        if ejoy_succ then
            product_infos = infos
            inited = true
            ET.publish('purchase_inited', CHANNEL)
            _ejoysdk.log("log uc product info succ")
        else
            _ejoysdk.log("log uc product info fail")
        end
    end)
end

function M.init(opt)
    pay_listener = opt.pay_listener
    auth_listener = opt.auth_listener

    if inited then
        return
    end

    ET.subscribe('login', login_handler)
    _ejoysdk.log("init UC")

    UNI.register_pay_listener(CHANNEL, function(succ, _order_id, _ext_params)
        pay_listener(succ)
    end)

    UNI.register_login_listener(CHANNEL, function(succ, info, _ext_params)
        if succ then
            local outsource = {
                platform = CHANNEL,
                ptoken = info.token,
                guest = false,
            }
            local ext = {}
            auth_listener(succ, outsource, ext)
        else
            auth_listener(succ, info)
        end
    end)
end

function M.login()
    UNI.login(CHANNEL, {})
end

function M.logout(cb)
    UNI.logout(CHANNEL)
    cb()
end

function M.merge_info(info, pinfo)
    return M.merge_helper(info, pinfo)
end

function M.simple_token()
    return false
end

function M.check_token(_outsource, _info)
    M.login()
end

function M.can_pay()
    return inited
end

function M.pay(_product_id, _count, _order_id, _body)
end

function M.product_list()
    return product_infos
end

function M.exit(cb)
    UNI.async_call(CHANNEL, ASYNC_EXIT, {}, nil, function(succ, ...)
        cb(succ, ...)
    end)
end

--[[
    info = {
        player_id = 'string',
        name = 'string',
        level = 'int',
        create_time = 'int',  -- 角色创建的时间， 单位秒
        zone_id = 'string',   -- 服务器id
        zone_name = 'string' -- 服务器名
    }
]]

function M.submit_role_data(info)
    UNI.cast(CHANNEL, CAST_SUBMIT_ROLE_DATA, info, nil)
end

M:is_implemented({"ACCOUNT", "PAY"})

return M
