local E = require "ejoysdk_lua.ejoysdk"
local ET = require "ejoysdk_lua.ejoysdk_topic"
local UNI = require "ejoysdk_lua.vendors.unisdk"
local Vendor = require "ejoysdk_lua.vendors.vendor"
local EG = require "ejoysdk_lua.ejoysdk_gangplank"

local CHANNEL = "XIAOMI"
local auth_listener = nil
local pay_listener = nil
local g_outsource = nil
local g_info = nil
local product_infos = {}
local user_info = nil
local inited = false

local M = Vendor:Inherit(CHANNEL)

local login_handler = function()
    if inited then
        return
    end

    _ejoysdk.log("get product info")

    EG.product_infos_base(CHANNEL, function(succ, infos)
        if succ then
            for k, v in pairs(infos) do
                if v.money % 100 == 0 then
                    v.money = math.floor(v.money / 100)
                    product_infos[k] = v
                else
                    _ejoysdk.log("error product: " .. k)
                end
            end
            E.log(product_infos)
            inited = true
            ET.publish('purchase_inited', CHANNEL)
        else
            _ejoysdk.log("log alipay product info fail")
        end
    end)
    _ejoysdk.log("get product info passed!")
end

function M.init(opt)
    auth_listener = opt.auth_listener
    pay_listener = opt.pay_listener

    if inited then
        return
    end

    ET.subscribe('login', login_handler)
    _ejoysdk.log("init XIAOMI")

    UNI.register_login_listener(CHANNEL, function(succ, info, _ext_params)
        if succ then
            g_outsource = {
                platform = info.channel,
                ptoken = info.token,
                pid = info.pid,
                guest = false,
                with = nil,
                with_account = nil,
            }
            g_info = {
                app_name = info.channel_product_code,
            }
            auth_listener(succ, g_outsource, g_info)
        else
            auth_listener(succ, info)
        end
    end)

    UNI.register_pay_listener(CHANNEL, function(succ, order_id, _ext_params)
        if succ then
            pay_listener(succ, order_id)
        else
            pay_listener(succ)
        end
    end)
end

function M.login()
    UNI.login(CHANNEL)
end

function M.logout(cb)
    g_outsource = nil
    g_info = nil
    UNI.logout(CHANNEL)
    cb()
end

function M.merge_info(info, pinfo)
    return M.merge_helper(info, pinfo)
end

function M.simple_token()
    return false
end

function M.check_token(_outsource, _info)
    M.login()
end

function M.can_pay()
    return inited and user_info
end

function M.pay(product_id, _count, order_id, _body)
    assert(product_infos[product_id], "product_id " .. tostring(product_id) .. " not found")

    local order_info = {
        product_id = product_id,
        money = product_infos[product_id].money,
    }
    for k, v in pairs(user_info) do
        order_info[k] = v
    end
    UNI.pay(CHANNEL, order_id, order_info)
end

function M.product_list()
    return product_infos
end

local INFO_KEYS = { "balance", "vip", "level", "guild", "name", "player_id", "server"}

function M.set_user_info(info)
    for _i, k in ipairs(INFO_KEYS) do
        assert(type(info[k]) == "string", "field: " .. k .. "not found or not string")
    end
    user_info = info
end

M:is_implemented({'ACCOUNT', 'PAY'})

return M
