local E = require "ejoysdk_lua.ejoysdk"
local ET = require "ejoysdk_lua.ejoysdk_topic"
local EG = require "ejoysdk_lua.ejoysdk_gangplank"
local UNI = require "ejoysdk_lua.vendors.unisdk"
local Vendor = require "ejoysdk_lua.vendors.vendor"
local JSON = require "ejoysdk_lua.ejoysdk_json"

local CHANNEL = "ZLONG"
local UNISDK_CHANNEL = "ZILONG"

local ASYNC_GET_PRODUCT_LIST = "ASYNC_GET_PRODUCT_LIST"
local ASYNC_INVESTIGATION = "ASYNC_INVESTIGATION"
local ASYNC_EXIT = "ASYNC_EXIT"
local ASYNC_SHARE = "ASYNC_SHARE"
local ASYNC_SET_EXT_DATA = "ASYNC_SET_EXT_DATA"
local ASYNC_CALL_WEB_VIEW = "ASYNC_CALL_WEB_VIEW"

local CAST_GAME_STARTED = "CAST_GAME_STARTED"
local CAST_GAME_EVENT = "CAST_GAME_EVENT"
local CAST_ADD_LOCAL_NOTIFICATION = "CAST_ADD_LOCAL_NOTIFICATION"
local CAST_SHOW_CUSTOMER_SERVICE_WEB = "CAST_SHOW_CUSTOMER_SERVICE_WEB"
local CAST_CLEAR_LOCAL_NOTIFICATION = "CAST_CLEAR_LOCAL_NOTIFICATION"
local CAST_CREATE_ROLE = "CAST_CREATE_ROLE"
local CAST_ROLE_LEVEL_UP = "CAST_ROLE_LEVEL_UP"
local CAST_GIFT_CODE = "CAST_GIFT_CODE"


local SYNC_GET_DEVICE_ID = "SYNC_GET_DEVICE_ID"
local SYNC_GET_PUSH_TOKEN = "SYNC_GET_PUSH_TOKEN"
local SYNC_GET_API_AVAILABLE = "SYNC_GET_API_AVAILABLE"
local SYNC_GET_CHANNEL_ID = "SYNC_GET_CHANNEL_ID"

-- iOS 特有
local ASYNC_RECORD_START = "ASYNC_RECORD_START"
local ASYNC_RECORD_STOP = "ASYNC_RECORD_STOP"

local CAST_ORDER_RETRY = "CAST_ORDER_RETRY"
local CAST_BORADCAST = "CAST_BORADCAST"

--[[
-- gangplank
    {
        "ext" => %{"opcode" => "10001"},
        "game" => "M1",
        "guest" => false,
        "platform" => "2010011001",
        "ptoken" => "{\"validateInfo\":\"Infinity\"}",
        "region" => "M1",
        "secret" => "HsWSL+d1e6A=",
        "server" => "s_4011",
        "with" => "ZLONG"
    }

-- product_list
    [{
        "goods_price":5,
        "goods_register_id":"com.zlongame.uc.50",
        "goods_number":1,
        "goods_id":"1001",
        "goods_icon":"",
        "goods_name":"50",
        "goods_describe":"",
        "type":0
    }]
]]


local M = Vendor:Inherit(CHANNEL)

M.EVT_SWITCH_USER_SUCC = "EVT_SWITCH_USER_SUCC"
M.EVT_SWITCH_USER_CANCEL = "EVT_SWITCH_USER_CANCEL"
M.EVT_SWITCH_USER_FAIL = "EVT_SWITCH_USER_FAIL"

local auth_listener = nil
local pay_listener = nil
local switch_listener = nil
local inited = false
local product_infos = {}

local function match_product_infos(admin_list, zilong_list)
    local ret = {}
    for _i, item in ipairs(zilong_list) do
        local target = admin_list[item.goods_id]
        if target then
            for k, v in pairs(item) do
                target[k] = v
            end
            ret[item.goods_id] = target
        end
    end
    return ret
end

local function get_product_list(cb)
    _ejoysdk.log("get zilong info")

    EG.product_infos_base(CHANNEL, function(ejoy_succ, infos)
        if ejoy_succ then
            UNI.async_call(UNISDK_CHANNEL, ASYNC_GET_PRODUCT_LIST, {}, nil, function(succ, zlong_product_list)
                if succ then
                    product_infos = match_product_infos(infos, zlong_product_list)
                    inited = true
                    ET.publish('purchase_inited', CHANNEL)
                    _ejoysdk.log("log zilong product info succ")
                else
                    _ejoysdk.log("log zilong product info fail")
                end
                cb(succ)
            end)
        else
            _ejoysdk.log("log ejoysdk product info fail")
            cb(ejoy_succ)
        end
    end)
end

local login_handler = function()
    if inited then
        return
    end

    get_product_list(function() end)
end

function M.init(opt)
    pay_listener = opt.pay_listener
    auth_listener = opt.auth_listener
    switch_listener = opt.switch_listener

    if inited then
        return
    end

    ET.subscribe('login', login_handler)
    _ejoysdk.log("init ZILONG")

    UNI.register_pay_listener(UNISDK_CHANNEL, function(succ, _order_id, _ext_params)
        pay_listener(succ, _order_id, _ext_params)
    end)

    UNI.register_login_listener(UNISDK_CHANNEL, function(succ, info, ext_params)
        if succ then
            local outsource = {
                platform = ext_params.channel_id,
                ptoken = ext_params.data,
                guest = false,
                with = CHANNEL,
                with_account = nil,
                ext = {
                    opcode =  ext_params.opcode
                }
            }
            local ext = {}
            auth_listener(succ, outsource, ext)
        else
            auth_listener(succ, info)
        end
    end)

    UNI.register_event_cb(UNISDK_CHANNEL, function(type_, body)
        opt.event_listener(type_, body, function(server, region)
            if type_ == M.EVT_SWITCH_USER_SUCC then
                local outsource = {
                    platform = body.channel_id,
                    ptoken = body.data,
                    guest = false,
                    with = CHANNEL,
                    with_account = nil,
                    ext = {
                        opcode =  body.opcode
                    }
                }
                switch_listener(server, region, outsource, {})
            end
        end)
    end)
end

function M.login()
    local nonce = _ejoysdk_crypt.base64encode(_ejoysdk_crypt.randomkey())
    UNI.login(UNISDK_CHANNEL, {nonce = nonce})
end

function M.logout(cb)
    UNI.logout(UNISDK_CHANNEL)
    cb()
end

function M.merge_info(info, pinfo)
    return M.merge_helper(info, pinfo)
end

function M.simple_token()
    return false
end

function M.check_token(_outsource, _info)
    M.login()
end

function M.can_pay()
    return inited
end

function M.pay(product_id, count, order_id, _body)
    local product = product_infos[product_id]
    assert(product, "product_id " .. tostring(product_id) .. " not found")

    local order_info = {
        name = product.goods_name,
        describe = product.goods_describe,
        amount = count,
        register_id = product.goods_register_id,
        product_id = product_id,
        money = product.goods_price,
    }
    UNI.pay(UNISDK_CHANNEL, order_id, order_info)
end

function M.product_list()
    return product_infos
end

function M.product_list_new(cb)
    get_product_list(function (succ)
        if succ then
            cb(succ, product_infos)
        else
            cb(succ, {})
        end
    end)
end

function M.investigation(cb)
    UNI.async_call(UNISDK_CHANNEL, ASYNC_INVESTIGATION, {}, nil, cb)
end

function M.game_started(info)
    UNI.cast(UNISDK_CHANNEL, CAST_GAME_STARTED, info)
end

function M.get_device_id()
    return UNI.sync_call(UNISDK_CHANNEL, SYNC_GET_DEVICE_ID, {})
end

function M.add_local_notification(title, content, date, hour, min)
    UNI.cast(UNISDK_CHANNEL, CAST_ADD_LOCAL_NOTIFICATION, {
            title = title,
            content = content,
            date = date,
            hour = hour,
            min = min,
        })
end

function M.clean_local_notification()
    UNI.cast(UNISDK_CHANNEL, CAST_CLEAR_LOCAL_NOTIFICATION, {})
end

function M.get_boolean_api_available(id)
    return UNI.sync_call(UNISDK_CHANNEL, SYNC_GET_API_AVAILABLE, {api=id})
end

function M.open_forum(cb)
    local forum_info = {
        data = {},
        data_type = "YsdkForum"
    }
    UNI.async_call(UNISDK_CHANNEL, ASYNC_SET_EXT_DATA, forum_info, nil, cb)
end

function M.open_vplayer(cb)
    local vplayer_info = {
        data = {},
        data_type = "YsdkVplayer"
    }
    UNI.async_call(UNISDK_CHANNEL, ASYNC_SET_EXT_DATA, vplayer_info, nil, cb)
end

function M.game_exit(cb)
    UNI.async_call(UNISDK_CHANNEL, ASYNC_EXIT, {}, nil, cb)
end

function M.open_customer_service()
    UNI.cast(UNISDK_CHANNEL, CAST_SHOW_CUSTOMER_SERVICE_WEB, {})
end

function M.create_role(info)
    UNI.cast(UNISDK_CHANNEL, CAST_CREATE_ROLE, info)
end

function M.role_level_up(info)
    UNI.cast(UNISDK_CHANNEL, CAST_ROLE_LEVEL_UP, info)
end

function M.gift_code(info)
    UNI.cast(UNISDK_CHANNEL, CAST_GIFT_CODE, info)
end

function M.do_share(info, cb)
    local share = {share=info}
    UNI.async_call(UNISDK_CHANNEL, ASYNC_SHARE, share, nil, cb)
end

function M.get_channel_id()
    return UNI.sync_call(UNISDK_CHANNEL, SYNC_GET_CHANNEL_ID, {})
end

function M.call_web_view(info, cb)
    return UNI.async_call(UNISDK_CHANNEL, ASYNC_CALL_WEB_VIEW, info, nil, cb)
end

if E.CONFIG.get_config('os') == 'android' then
    function M.get_push_token()
        return UNI.sync_call(UNISDK_CHANNEL, SYNC_GET_PUSH_TOKEN, {})
    end

    function M.game_event(id, remark)
        UNI.cast(UNISDK_CHANNEL, CAST_GAME_EVENT, {
                event_id = id,
                remark = remark or "",
            })
    end
else
    function M.record_start(cb)
        UNI.async_call(UNISDK_CHANNEL, ASYNC_RECORD_START, {}, nil, cb)
    end

    function M.record_stop(cb)
        UNI.async_call(UNISDK_CHANNEL, ASYNC_RECORD_STOP, {}, nil, cb)
    end

    function M.order_retry()
        UNI.cast(UNISDK_CHANNEL, CAST_ORDER_RETRY, {})
    end

    function M.boradcast()
        UNI.cast(UNISDK_CHANNEL, CAST_BORADCAST, {})
    end

    function M.game_event(id, remark)
        local remark_str = remark and JSON:encode(remark) or "{}"
        UNI.cast(UNISDK_CHANNEL, CAST_GAME_EVENT, {
                event_id = id,
                remark = remark_str,
            })
    end
end

M:is_implemented({"ACCOUNT", "PAY", "SHARE"})

return M
